import boto3
import json


def configure_iam():
    iam = boto3.client("iam")

    with open("config-iam.json") as file:
        json_data = json.load(file)
    namespace = json_data["namespace"]
    policies = json_data["policies"]
    iam_user_purpose = json_data["purpose"]
    user_exists = bool(json_data.get("arn"))

    policies_to_attach = list()
    policies_to_update = list()

    for policy in policies:
        description = policy.get("description")
        created = bool(policy.get("arn"))

        if not created:
            path_to_policy = policy["path"]
            purpose = policy["purpose"]

            with open(path_to_policy) as file:
                policy_json = file.read()
            response = iam.create_policy(
                PolicyName=f"{namespace}-{purpose}",
                PolicyDocument=policy_json,
                Description=description,
            )
            arn = response["Policy"]["Arn"]
            policy["arn"] = arn
            policies_to_attach.append(arn)
        else:
            policies_to_update.append(policy)

    username = f"{namespace}-{iam_user_purpose}"

    if not user_exists:
        response = iam.create_user(
            UserName=username,
        )
        arn = response["User"]["Arn"]
        json_data["arn"] = arn

    for policy_arn in policies_to_attach:
        iam.attach_user_policy(UserName=username, PolicyArn=policy_arn)
    for policy in policies_to_update:
        policy_arn = policy["arn"]
        results = iam.list_policy_versions(PolicyArn=policy_arn)
        versions = results["Versions"]
        for idx, version in enumerate(versions):
            version_id = version["VersionId"]
            if idx == 4:
                iam.delete_policy_version(PolicyArn=policy_arn, VersionId=version_id)

        with open(policy["path"]) as file:
            policy_json = file.read()
        response = iam.create_policy_version(
            PolicyArn=policy_arn, PolicyDocument=policy_json, SetAsDefault=True
        )

    with open("config-iam.json", "w") as file:
        json.dump(json_data, file, indent=2)


import os
from typing import Callable
import django

from django.conf import settings

from django_configuration_management.aws_utils import pull_aws_config_data
from django_configuration_management.utils import (
    load_env,
    normalize_config_data,
)
from django_configuration_management.yml_utils import yml_to_dict


def get_config(environment: str, dotenv_required=True):
    try:
        load_env(environment)
    except AssertionError as error:
        if dotenv_required:
            raise error

    local_secrets, aws_secrets = yml_to_dict(environment)

    normalized_local_secrets = normalize_config_data(local_secrets)
    pulled_aws_secrets = pull_aws_config_data(aws_secrets)

    return {**normalized_local_secrets, **pulled_aws_secrets}


def inject_config(environment: str, settings_module: settings, dotenv_required=True):
    config = get_config(environment, dotenv_required)

    for key, value in config.items():
        setattr(settings_module, key, value)


def load_django(
    path_to_settings_module: str,
    pre_setup_hook: Callable = lambda *args, **kwargs: ...,
    pre_setup_hook_args: list = [],
    pre_setup_hook_kwargs: dict = {},
    post_setup_hook: Callable = lambda *args, **kwargs: ...,
    post_setup_hook_args: list = [],
    post_setup_hook_kwargs: dict = {},
):
    """
    For use in scripts that invokved via manage.py, e.g., lambda environments, non-management command CLI scripts, etc
    """
    pre_setup_hook(*pre_setup_hook_args, **pre_setup_hook_kwargs)
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", path_to_settings_module)
    django.setup()
    post_setup_hook(*post_setup_hook_args, **post_setup_hook_kwargs)


import os
from getpass import getpass
from pathlib import Path

from dotenv import load_dotenv


from django_configuration_management.secrets import decrypt_value, encrypt_value
from django_configuration_management.yml_utils import validate_key_name


def load_env(environment: str):
    env_path = Path(".") / f".env-{environment}"
    if os.path.isfile(env_path):
        load_dotenv(dotenv_path=env_path, verbose=True)
        return

    raise AssertionError(
        f"env not found: {env_path}. This file is required and must contain your ENC_KEY"
    )


def gather_user_input():
    key_name = input("Enter key name: ")

    validate_key_name(key_name)

    key_value = getpass("Enter key value: ")

    return key_name, encrypt_value(key_value)


def normalize_config_data(data: dict):
    normalized = dict()
    for key, meta in data.items():
        if type(meta) == dict:
            value = meta["value"]
            normalized[key] = decrypt_value(value)
        else:
            normalized[key] = meta
    return normalized



import requests
import json
import subprocess
import re

from base64 import b64encode
from nacl import encoding, public


def get_remote_repo():
    result = subprocess.Popen("git remote -v", shell=True, stdout=subprocess.PIPE)

    if result is None:
        print("Local repository could not be determined. Exiting...")
        quit()

    repo_result_w_byte = result.stdout.read()
    stripped = repo_result_w_byte.decode(
        "utf-8-sig"
    ).rstrip()  # Remove byte order mark and new line

    return stripped


def owner_and_repo():
    repo_url = get_remote_repo()
    owner_and_repo = re.search(r"[A-Za-z0-9_-]+/[A-Za-z0-9_-]+.git", repo_url).group()
    owner_repo_array = owner_and_repo.split("/")
    owner = owner_repo_array[0]
    repo_name = owner_repo_array[1].split(".")[0]

    if owner is None:
        print(f"Owner for repository could not be found. Exiting...")
        quit()

    if repo_name is None:
        print(f"Repository name could not be determined. Exiting...")
        quit()
    return [owner, repo_name]


def repository_info(github_access_token, owner, repo_name):
    # Get a repository public key
    headers = {"Authorization": "token " + github_access_token}
    repo_public_key_string = requests.get(
        f"https://api.github.com/repos/{owner}/{repo_name}/actions/secrets/public-key",
        headers=headers,
    ).text
    repo_public_key_dict = json.loads(repo_public_key_string)

    assert (
        "key" in repo_public_key_dict
    ), f"Repo {repo_name} with owner {owner} encountered {repo_public_key_dict['message']}"

    repo_key = repo_public_key_dict["key"]
    key_id = repo_public_key_dict["key_id"]
    return [repo_key, key_id]


def encrypt_secret(user_public_key: str, secret_value: str):
    """
    Encrypt a Unicode string using the public key.
        https://docs.github.com/en/rest/reference/actions#create-or-update-a-repository-secret
    """
    public_key = public.PublicKey(
        user_public_key.encode("utf-8"), encoding.Base64Encoder()
    )
    sealed_box = public.SealedBox(public_key)
    encrypted = sealed_box.encrypt(secret_value.encode("utf-8"))
    return b64encode(encrypted).decode("utf-8")


def push_secrets_to_github(
    owner, repo_name, key_id, secrets_dictionary, github_access_token
):
    for secret_key in secrets_dictionary:
        headers = {
            "accept": "application/vnd.github.v3+json",
            "Authorization": "token " + github_access_token,
        }
        payload = {
            "owner": owner,
            "repo": repo_name,
            "secret_name": secret_key,
            "encrypted_value": secrets_dictionary[secret_key],
            "key_id": key_id,
        }
        url = f"https://api.github.com/repos/{owner}/{repo_name}/actions/secrets/{secret_key}"
        print(f"Pushing secrets to url: {url}")
        try:
            response = requests.put(url, headers=headers, data=json.dumps(payload))
        except Exception as e:
            print(f"Failed to push secrets to remote repository: {e}")



import click
import os
import json

from django_configuration_management.aws_utils import pull_aws_config_data
from django_configuration_management.secrets import (
    decrypt_value,
    encrypt_value,
    generate_fernet_key,
)
from django_configuration_management.utils import gather_user_input, load_env
from django_configuration_management.github_utils import (
    owner_and_repo,
    repository_info,
    encrypt_secret,
    push_secrets_to_github,
)
from django_configuration_management.yml_utils import dict_to_yml, yml_to_dict

from django_configuration_management.iam_script import configure_iam


@click.command("upsert_secret")
@click.option("-e", "--environment", default="development", help="Your environment")
def upsert_secret(environment):
    load_env(environment)

    data, _ = yml_to_dict(environment, skip_required_checks=True)
    key_name, key_value = gather_user_input()

    data[key_name] = {"value": key_value, "secret": True}

    dict_to_yml(data, environment)


@click.command("reveal_secrets")
@click.option("-e", "--environment", default="development", help="Your environment")
def reveal_secrets(environment):
    load_env(environment)

    local_secrets, aws_secrets = yml_to_dict(environment, skip_required_checks=True)

    for key, meta in local_secrets.items():
        # Skip non-secret values
        if type(meta) != dict:
            continue

        value = meta["value"]
        secret = decrypt_value(value)
        print(f"{key}={secret}")

    for key, secret in pull_aws_config_data(aws_secrets).items():
        print(f"{key}={secret}")


@click.command("generate_key")
def generate_key():
    key = generate_fernet_key()

    print(f"Your key is: \n\n{key}\n")
    print(
        "Please store this in whichever .env-[environment] file you generated it for under the variable ENC_KEY"
    )


@click.command("reencrypt")
@click.option("-e", "--environment", default="development", help="Your environment")
@click.option(
    "-k",
    "--new-key",
    help="The new key with which you'd like to re-encrypt your secrets",
)
def reencrypt(environment, new_key):
    load_env(environment)

    if not new_key:
        new_key = generate_fernet_key()

    data, _ = yml_to_dict(environment, skip_required_checks=True)

    for key, meta in data.items():
        # Skip non-secret values
        if type(meta) != dict:
            continue

        value = meta["value"]
        decrypted_value = decrypt_value(value)
        meta["value"] = encrypt_value(decrypted_value, enc_key=new_key)

        data[key] = meta

    dict_to_yml(data, environment)

    print(f"Your new key is: \n\n{new_key}\n")
    print(
        f"config-{environment}.yaml has been re-encrypted. Please be sure to update your .env-{environment} with the new key you used!"
    )


@click.command("github_secrets")
def github_secrets():
    """
    1. Get github public auth key from local env
    2. Get current repo name based on current dir
    3. Get repo public key
    4. Loop through `config-github.json` file and:
        - Ask to provide secret value for specific key
        - Encrypt provided secret
        - Save to dictionary
    5. Push secrets to github repo
    """
    print(
        "WARNING! This is an expiremental feature and is likely to change with future releases"
    )
    # check for the file right away
    with open(
        "config-github.json",
    ) as github_config_file:
        github_config = json.load(github_config_file)

    key = "GITHUB_ACCESS_TOKEN"
    github_access_token = os.getenv(key)

    assert github_access_token, f"Environment variable {key} not set"

    owner_repo_array = owner_and_repo()
    owner = owner_repo_array[0]
    repo_name = owner_repo_array[1]

    repo_key_info = repository_info(github_access_token, owner, repo_name)
    repo_key = repo_key_info[0]
    key_id = repo_key_info[1]

    user_provided_secrets = {}

    # Prompt user for secret and encrypt
    for key in github_config:
        prompt = f"Please provide the secret for key: {key}"
        repo_secret = input(prompt + " (or press enter to skip):\n")
        if not repo_secret:
            continue
        encrypted_secret = encrypt_secret(repo_key, repo_secret)
        user_provided_secrets[key] = encrypted_secret

    push_secrets_to_github(
        owner, repo_name, key_id, user_provided_secrets, github_access_token
    )


@click.command("iam_role")
def iam_role():
    print(
        "WARNING! This is an untested, expiremental feature. You use this at your own peril!"
    )
    configure_iam()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_configuration_management']

package_data = \
{'': ['*']}

install_requires = \
['PyGithub>=1.54.1,<2.0.0',
 'PyNaCl>=1.4.0,<2.0.0',
 'boto3>=1.17.27,<2.0.0',
 'cryptography>=3.3.1,<4.0.0',
 'django>=3.1.4,<4.0.0',
 'python-dotenv>=0.15.0,<0.16.0',
 'pyyaml>=5.3.1,<6.0.0']

entry_points = \
{'console_scripts': ['generate_key = '
                     'django_configuration_management.cli:generate_key',
                     'github_secrets = '
                     'django_configuration_management.cli:github_secrets',
                     'iam_role = django_configuration_management.cli:iam_role',
                     'reencrypt = '
                     'django_configuration_management.cli:reencrypt',
                     'reveal_secrets = '
                     'django_configuration_management.cli:reveal_secrets',
                     'upsert_secret = '
                     'django_configuration_management.cli:upsert_secret']}

setup_kwargs = {
    'name': 'django-configuration-management',
    'version': '0.3.7',
    'description': 'A merge conflict-less solution to committing an encrypted configuration to the repo with secrets and non-secrets side-by-side.',
    'long_description': '# Quick start\n\nThis package features an opinionated configuration management system, focused on combining both secret\nand non-secret keys in the same configuration file. The values for secret keys are encrypted and can\nbe committed to the repo, but since each key is separated on a line-by-line basis, merge conflicts\nshouldn\'t cause much trouble.\n\nThis package is intended to be used with a django project, though it\'s currently not making use\nof any Django specific features.\n\nNeedless to say, this is in very early development.\n\n## Install\n\n`pip install django-configuration-management`\n\n## cli\n\n### Generate a key\n\nIn a terminal, enter:\n\n```bash\ngenerate_key\n```\n\nFollow the instructions printed to the console. For example, if you\'re setting up a production configuration,\nmake a file called `.env-production` in the root of your django project. Inside of it, save the key generated\nabove to a variable called `ENC_KEY`.\n\n### Upsert a secret\n\nTo insert or update a secret, enter:\n\n```bash\nupsert_secret --environment <your environment>\n```\n\nAnd follow the prompts.\n\n### Insert a non-secret\n\nSimply open the .yml file for the generated stage (the naming scheme is `config-<environment>.yaml`),\nand insert a row. It should look like this:\n\n```yaml\nUSERNAME: whatsup1994 # non-secret\nPASSWORD:\n  secret: true\n  value: gAAAAABf2_kxEgWXQzJ0SlRmDy6lbXe-d3dWD68W4aM26yiA0EO2_4pA5FhV96uMWCLwpt7N6Y32zXQq-gTJ3sREbh1GOvNh5Q==\n```\n\n### Insert an AWS Secret Manager secret\n\nAdd a row where the key name is your secret name in AWS Secret Manager, with a sub-key value pair of\n`use_aws: true`. It should look like this:\n\n```yaml\nsecrets/integrations/aws_secret:\n  use_aws: true\n  secret_keys:\n    - AWS_SECRET\n```\n\nThe secret in your AWS instance will need to conform to the naming patterns used elsewhere in\nthis repo, e.g., a properly structured AWS Secret Manager secret will look like\n\n```json\n{\n  "AWS_SECRET": "I\'m a secret"\n}\n```\n\nThe keys of this object will be translated into python variables for the Django settings\nmodule in much the same way the keys of the local yaml will be, but you must explicitly\ncall out which keys you want to load by specifying the attribute `secret_keys`.\n\nYour AWS credentials must also be set up correctly to make API calls to AWS.\n\n### Manually editing the file\n\nYou can change the values of non-secrets by hand, as well as the keynames, but clearly you must\nnot change the value of secrets by hand, as they\'re encrypted. Changing the order of any of the\nkeys is perfectly fine.\n\n### Print secrets to the console\n\nTo show the decrypted values of all the secrets in the console, enter:\n\n```bash\nreveal_secrets --environment <your-environment>\n```\n\n### Re-encrypt a config file\n\nTo re-encrypt all secret values for a given environment\'s config file, pass\n\n```bash\nreencrypt --environment <your-environment> --new-key <your-new-key>\n```\n\nIf you do not provide a key, a new one will be generated for you.\n\n## Configuring repository secrets from cli using `github_secrets`\n\nTo set secrets on a remote repository:\n\n1. Create a `GITHUB_ACCESS_TOKEN` variable in your local shell environment.\n   This variable should contain your github personal access token\n   (https://docs.github.com/en/github/authenticating-to-github/creating-a-personal-access-token).\n2. `cd` into the local repository that you would like to set secrets for.\n3. Create a new file named `config-github.json` and add a JSON object where each\n   key is the name of the secret you would like to add to the remote repository.\n   The value can be an empty string or a brief description about the key.\n   e.g.\n   ```json\n   {\n     "AWS_ACCESS_KEY_ID": "Description",\n     "AWS_SECRET_ACCESS_KEY": "Description"\n   }\n   ```\n4. Run `github_secrets` to start running the script.\n5. This will run the script and prompt you to enter a value for each secret key in\n   the `config-github.json` file.\n6. You can press `enter` to skip providing a value for any secret.\n7. Once you have either provided a value or skipped all secrets in the `config-github.json` file,\n   the script will push the secrets to the remote repository of the current working directory.\n\n## Extras\n\nIn the root of your django project, you can create a file called `config-required.json`.\n\nThe JSON object can be a list or a dictionary. This is useful for validating the presence of your\nkeys on start-up.\n\nValidating secrets that come from AWS Secret Manager is done implicitly since you must specify\na `secret_keys` attribute in your .yaml. This is needed so that only AWS secrets that are\nexplicitly called out are loaded into your Django settings/Python environment.\n\n## Settings\n\nThere are two ways to use this library, if you don\'t mind a little magic, you can\nsimply inject the config by importing the following function in your django settings file,\nand passing in the current module.\n\n```python\n# settings.py\nfrom django_configuration_management import inject_config\n\n# development is the environment name\ninject_config("development", sys.modules[__name__])\n```\n\nSee the example project for a demonstration of this.\n\nIf you want more verbosity, you can import the following function which will return\nthe config as a normalized dictionary that\'s flat and has all secrets decrypted.\n\n```python\n# settings.py\nfrom django_configuration_management import get_config\n\n# config = {"USERNAME": "helloworld", "PASSWORD": "im decrypted}\nconfig = get_config("development")\n\nUSERNAME = config["USERNAME"]\n# ...\n```\n\n### Using without a .env\n\nIf you want to skip using the .env, you can set the optional argument `dotenv_required` to `False`\nwhen invoking either of the above two methods. Doing so means it then becomes your responsibility\nto load an environment variable called `ENC_KEY` that stores the relevant encryption key for the\nstage you\'re trying to load.\n\n```python\n# settings.py\nfrom django_configuration_management import get_config\n\n# Will error out if you didn\'t load ENC_KEY correctly\nconfig = get_config("development", dotenv_required=False)\n```\n\n---\n\nThis project uses [poetry](https://python-poetry.org/) for dependency management\nand packaging.\n',
    'author': 'Alex Drozd',
    'author_email': 'drozdster@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/brno32/django-configuration-management',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import yaml

from django_configuration_management.validation_utils import (
    read_required_vars_file,
    validate_key_name,
)


def _validate_yml(data, skip_required_checks=False):
    if not skip_required_checks:
        _check_required_keys(data)

    for key, meta in data.items():
        validate_key_name(key)
        if type(meta) == dict:
            secret = meta.get("secret")
            if secret is not None:
                assert (
                    secret
                ), f"{key} is structured like a secret value, but you've marked it as 'secret: false'. The value of this key can simply be the plain text value."
                assert (
                    type(meta.get("value")) == str
                ), f"{key} has an invalid row. Missing 'value'"
            else:
                raise AssertionError(f"{key} has an invalid row. Missing 'secret_name'")


def _check_required_keys(data):
    required_vars = read_required_vars_file()
    if not required_vars:
        return

    missing_keys = []
    for key in required_vars:
        if key not in data:
            missing_keys.append(key)

    assert (
        len(missing_keys) < 1
    ), f"The following keys are required. {missing_keys}. Halting"


def yml_to_dict(environment: str, skip_required_checks=False):
    try:
        with open(f"config-{environment}.yaml", "r") as yml:
            loaded: dict = yaml.safe_load(yml)
    except FileNotFoundError:
        loaded = {}

    local_secrets, aws_secrets = separate_aws_secrets(loaded)
    _validate_yml(local_secrets, skip_required_checks)
    return local_secrets, aws_secrets


def dict_to_yml(data: dict, environment: str) -> str:
    with open(f"config-{environment}.yaml", "w+") as yml:
        dumped = yaml.dump(data, yml)
    return dumped


def separate_aws_secrets(config: dict):
    aws = dict()
    local = dict()
    for key, meta in config.items():
        is_aws = type(meta) == dict and meta.get("use_aws")
        if is_aws:
            aws[key] = meta
        else:
            local[key] = meta
    return local, aws
